# -*- coding: utf-8 -*-


__all__ = ["get_poly_area",
           "get_poly_bounds",
           "get_poly_hw"]


def get_poly_area(poly1):
    """ Calcualte the number of pixels the polygon covered.

    Parameters
    -------
    poly1 : Polygon
        contour with shapely polygon format

    Returns
    -------
    area_val : int
        number of pixels inside the contour

    """

    area_val = poly1.area

    return area_val


def get_poly_bounds(poly1):
    """ Find the bounds of the Polygon.

    Parameters
    -------
    poly1 : Polygon
        contour with shapely polygon format

    Returns
    -------
    min_h : int
        minimum y coordinate of polygon
    min_w : int
        minimum x coordinate of polygon
    max_h : int
        maximum y coordinate of polygon
    max_w : int
        maximum x coordinate of polygon

    """

    min_x, min_y, max_x, max_y = poly1.bounds
    min_h, min_w = min_y, min_x
    max_h, max_w = max_y, max_x

    return min_h, min_w, max_h, max_w


def get_poly_hw(poly1):
    """ Find height and width of the polygon.

    Parameters
    -------
    poly : Polygon
        contour with shapely polygon format

    Returns
    -------
    poly_h : int
        height of the polygon
    poly_w : int
        width of the polygon

    """

    min_h, min_w, max_h, max_w = get_poly_bounds(poly1)

    poly_h = max_h - min_h + 1
    poly_w = max_w - min_w + 1

    return poly_h, poly_w



# -*- coding: utf-8 -*-

import numpy as np
import cv2


__all__ = ["get_cnt_area",
           "get_cnt_aspect_ratio",
           "get_cnt_solidity"]


def get_cnt_area(cnt1):
    """ Calcualte the number of pixels contour covered.

    Parameters
    -------
    cnt1 : np.array
        contour with opencv format

    Returns
    -------
    area_val : int
        number of pixels inside the contour

    """

    area_val = cv2.contourArea(cnt1.astype(np.float32))

    return area_val


def get_cnt_aspect_ratio(cnt1):
    """ Calcualte the aspect ratio of contour.

    Parameters
    -------
    cnt1 : np.array
        contour with opencv format

    Returns
    -------
    aspect_ratio : float
        contour aspect ratio

    """

    _, _, w, h = cv2.boundingRect(cnt1.astype(np.float32))
    aspect_ratio = w * 1.0 / h

    return aspect_ratio


def get_cnt_solidity(cnt1):
    """ Calcualte the solidity of contour.

    Parameters
    -------
    cnt : np.array
        contour with opencv format

    Returns
    -------
    solidity_val : float
        solidity value of contour

    """

    cnt1 = cnt1.astype(np.float32)
    area = cv2.contourArea(cnt1)
    hull = cv2.convexHull(cnt1)
    hull_area = cv2.contourArea(hull)

    solidity_val = area * 1.0 / hull_area

    return solidity_val



# -*- coding: utf-8 -*-

import os

BASE_PATH = os.path.abspath(os.path.dirname(__file__))
PKG_NAME = os.path.basename(BASE_PATH)


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration(PKG_NAME, parent_package, top_path)

    # Add subpackage
    config.add_subpackage('cnt')
    config.add_subpackage('fea')
    config.add_subpackage('img')
    config.add_subpackage('poly')
    config.add_subpackage('rela')
    config.add_subpackage('transform')

    return config


if __name__ == "__main__":
    from numpy.distutils.core import setup

    config = configuration(top_path='').todict()
    setup(**config)



# -*- coding: utf-8 -*-

import os

BASE_PATH = os.path.abspath(os.path.dirname(__file__))
MODULE_NAME = os.path.basename(BASE_PATH)


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration(MODULE_NAME, parent_package, top_path)

    return config


if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(maintainer='Pingjun Chen',
          maintainer_email='chenpingjun@gmx.com',
          description='Polygon utilities.',
          url='https://github.com/PingjunChen/pycontour',
          license='MIT',
          **(configuration(top_path='').todict())
          )



# -*- coding: utf-8 -*-

import numpy as np
import cv2
from skimage import filters
from skimage import img_as_ubyte

from ..cv2_transform import cv_cnt_to_np_arr
from ..img import mask

__all__ = ["smooth_cnt", ]


def smooth_cnt(np_arr, sigma=5):
    """ Smooth the contour.

    Parameters
    -------
    np_arr : np.array
        contour with standard numpy 2d array format
    sigma: int
        gaussian smoothing kernel

    Returns
    -------
    smooth_arr: np.array
        smoothed contour

    """

    min_h, min_w = np.min(np_arr[0]), np.min(np_arr[1])
    np_arr[0] -= min_h
    np_arr[1] -= min_w

    cnt_mask = mask.build_cnt_mask(np_arr)
    smooth_mask = filters.gaussian(cnt_mask, sigma=sigma)
    bin_mask = smooth_mask > 0.99

    _, cnts, _ = cv2.findContours(img_as_ubyte(bin_mask),
                                  mode=cv2.RETR_EXTERNAL,
                                  method=cv2.CHAIN_APPROX_NONE)
    if len(cnts) != 1:
        raise AssertionError("Contour number error...")

    smooth_cnt = cnts[0]
    smooth_arr = cv_cnt_to_np_arr(smooth_cnt)
    smooth_arr[0] += min_h
    smooth_arr[1] += min_w

    return smooth_arr



# -*- coding: utf-8 -*-

import math
import numpy as np

__all__ = ["rotate_cnt", ]

def rotate_cnt(np_arr, angle):
    """ Rotate contour clockwise by radian angle around contour center

    Parameters
    -------
    np_arr : np.array
        contour with standard numpy 2d array format
    angle : int or float
        angle in radian, range from 0 to 360

    Returns
    -------
    rot_cnt: np.array
        rotated contour

    """

    rotate_cnt = np.zeros_like(np_arr)
    cen_h = np.mean(np_arr[0, :])
    cen_w = np.mean(np_arr[1, :])

    rad = math.radians(angle)
    for ip in np.arange(np_arr.shape[1]):
        cnt_h, cnt_w = np_arr[0, ip], np_arr[1, ip]
        new_h = cen_h + math.sin(rad) * (cnt_w - cen_w) + math.cos(rad) * (cnt_h - cen_h)
        new_w = cen_w + math.cos(rad) * (cnt_w - cen_w) - math.sin(rad) * (cnt_h - cen_h)

        rotate_cnt[0, ip] = int(new_h)
        rotate_cnt[1, ip] = int(new_w)

    return rotate_cnt



# -*- coding: utf-8 -*-

from ..poly_transform import np_arr_to_poly


__all__ = ["contour_intersects",
           "construct_intersection_polygon",
           "contour_contains"]


def contour_intersects(np_arr1, np_arr2):
    """ Determine two contours are intersected or not.

    Parameters
    -------
    np_arr1 : np.array
        contour with standard numpy 2d array format
    np_arr2 : np.array
        contour with standard numpy 2d array format

    Returns
    -------
    inter_flag : bool
        check if two contours are intersected or not.

    """

    # Construct polygon from numpy array
    poly1 = np_arr_to_poly(np_arr1)
    poly2 = np_arr_to_poly(np_arr2)

    # Get convex_hull of polygon
    poly1 = poly1.convex_hull
    poly2 = poly2.convex_hull

    inter_flag = poly1.intersects(poly2)

    return inter_flag


def construct_intersection_polygon(np_arr1, np_arr2):
    """Construct polygon from the intersection part of two contours.

    Parameters
    -------
    np_arr1 : np.array
        contour with standard numpy 2d array format
    np_arr2 : np.array
        contour with standard numpy 2d array format

    Returns
    -------
    inter_poly : Polygon or None
        intersection polygon of two given polygons

    """

    inter_flag = contour_intersects(np_arr1, np_arr2)
    if inter_flag == False:
        return None

    # Construct polygon from numpy array
    poly1 = np_arr_to_poly(np_arr1)
    poly2 = np_arr_to_poly(np_arr2)

    # Get convex_hull of polygon
    poly1 = poly1.convex_hull
    poly2 = poly2.convex_hull

    inter_poly = poly1.intersection(poly2)

    return inter_poly


def contour_contains(np_arr1, np_arr2):
    """ Determine if contour of np_arr1 contains contour of np_arr2.

    Parameters
    -------
    np_arr1 : np.array
        contour with standard numpy 2d array format
    np_arr2 : np.array
        contour with standard numpy 2d array format

    Returns
    -------
    in_status : bool
        check if polygon 1 contains polygon 2 or not

    """

    # Construct polygon from numpy array
    poly1 = np_arr_to_poly(np_arr1)
    poly2 = np_arr_to_poly(np_arr2)

    # Get convex_hull of polygon
    poly1 = poly1.convex_hull  # Get convex hull of poly
    poly2 = poly2.convex_hull

    in_status = poly1.contains(poly2)

    return in_status



# -*- coding: utf-8 -*-

import os, sys

__all__ = ["__version__"]

__version__ = '1.4.0'

from . import cnt
from . import fea
from . import img
from . import poly
from . import rela
from . import transform
from .coor_transform import *
from .cv2_transform import *
from .poly_transform import *



# -*- coding: utf-8 -*-

import numpy as np
from shapely.geometry import Polygon, box

from .coor_transform import swap_wh
from .coor_transform import np_arr_to_point_list
from .coor_transform import point_list_to_np_arr


__all__ = ["np_arr_to_poly",
           "point_list_to_poly",
           "bbox_to_poly",
           "poly_to_np_arr"]


def np_arr_to_poly(np_arr):
    """ Using numpy 2d array ([0]-h, [1]-w) to construct polygon.

    Parameters
    -------
    np_arr : np.array
        contour with standard numpy 2d array format

    Returns
    -------
    poly : Polygon
        contour with shapely polygon format

    """

    np_arr = swap_wh(np_arr)
    point_list = np_arr_to_point_list(np_arr)
    poly = Polygon(point_list)

    return poly


def point_list_to_poly(point_list):
    """ Using point list to construct polygon.

    Parameters
    -------
    point_list : list
        list of point set ([0]-h, [1]-w)

    Returns
    -------
    poly : Polygon
        contour with shapely polygon format

    """

    wh_point_list = []
    for ind in np.arange(len(point_list)):
        wh_point_list.append((point_list[ind][1], point_list[ind][0]))

    poly = Polygon(wh_point_list)

    return poly


def bbox_to_poly(min_h, min_w, max_h, max_w):
    """ Using bounding box to construct polygon.

    Parameters
    -------
    min_h : int
        minimum y coordinate of polygon
    min_w : int
        minimum x coordinate of polygon
    max_h : int
        maximum y coordinate of polygon
    max_w : int
        maximum x coordinate of polygon

    Returns
    -------
    poly : Polygon
        contour with shapely polygon format

    """

    poly = box(min_w, min_h, max_w, max_h)

    return poly


def poly_to_np_arr(poly):
    """ Convert shapely Polygon to numpy 2d array ([0]-h, [1]-w).

    Parameters
    -------
    poly : Polygon
        contour with shapely polygon format

    Returns
    -------
    cnt_arr : np.array
        contour with standard numpy 2d array format

    """

    x_coors, y_coors = poly.exterior.coords.xy
    x_coors = x_coors[:-1].tolist()
    y_coors = y_coors[:-1].tolist()
    point_list = [(y, x) for x, y in zip(x_coors, y_coors)]
    cnt_arr = point_list_to_np_arr(point_list)

    return cnt_arr



# -*- coding: utf-8 -*-

import numpy as np
import cv2

from ..cv2_transform import np_arr_to_cv_cnt
from ..poly_transform import np_arr_to_poly
from ..poly import get_poly_bounds
from ..transform import shift_cnt


__all__ = ["build_cnt_mask",
           "cnt_mask_img",
           "cnt_mask_sub_img"]


def build_cnt_mask(np_arr, mask_size=None):
    """ Build an exterior rectangle mask based on contour

    Parameters
    -------
    np_arr : np.array
        contour with standard numpy 2d array format
    mask_size: None, scalar, list, or tuple
        desired mask size for contour

    Returns
    -------
    mask: np.array
        contour exterior rectangle mask

    """

    poly = np_arr_to_poly(np_arr)
    min_h, min_w, max_h, max_w = get_poly_bounds(poly)
    min_h, min_w = int(min_h), int(min_w)
    max_h, max_w = int(max_h), int(max_w)

    cnt_height = max_h - min_h + 1
    cnt_width = max_w - min_w + 1
    if mask_size == None:
        mask_height = cnt_height
        mask_width = cnt_width
    elif np.isscalar(mask_size):
        if not (mask_size >= cnt_height and mask_size >= cnt_width):
            raise AssertionError("given size too small")
        mask_height = mask_size
        mask_width = mask_size
    elif isinstance(mask_size, list) or isinstance(mask_size, tuple):
        if len(mask_size) == 1:
            if not (mask_size[0] >= cnt_height and mask_size[0] >= cnt_width):
                raise AssertionError("given size too small")
            mask_height = mask_size[0]
            mask_width = mask_size[0]
        elif len(mask_size) == 2:
            if not (mask_size[0] >= cnt_height and mask_size[1] >= cnt_width):
                raise AssertionError("given size too small")
            mask_height = mask_size[0]
            mask_width = mask_size[1]
        else:
            raise Exception("Not proper size")
    else:
        raise Exception("Not proper size")

    mask = np.zeros((mask_height, mask_width), np.uint8)
    mask_start_h = int(np.floor((mask_height - cnt_height) / 2.0))
    mask_start_w = int(np.floor((mask_width - cnt_width) / 2.0))

    # Fill using cv2
    shift_h = mask_start_h - min_h
    shift_w = mask_start_w - min_w
    shift_arr = shift_cnt(np_arr, shift_h, shift_w)
    cv_cnt = np_arr_to_cv_cnt(shift_arr)
    cv2.drawContours(mask, [cv_cnt], 0, 255, -1)

    return mask


def cnt_mask_img(img, np_arr):
    """ Mask image by contour

    Parameters
    -------
    img : np.array
        image represented by numpy array
    np_arr: np.array
        contour with standard numpy 2d array format

    Returns
    -------
    mask_img: np.array
        masked image with only contour covered region

    """

    mask = np.zeros_like(img, dtype=np.uint8)
    mask = np.ascontiguousarray(mask)

    cv_cnt = np_arr_to_cv_cnt(np_arr)
    if len(img.shape) == 2:
        cv2.drawContours(mask, [cv_cnt], 0, 1, -1)
    elif len(img.shape) == 3 and img.shape[2] == 3:
        cv2.drawContours(mask, [cv_cnt], 0, [1, 1, 1], -1)
    else:
        raise NotImplementedError("Image shape not supported")

    mask_img = img * mask

    return mask_img


def cnt_mask_sub_img(img, np_arr):
    """ Mask sub image using contour

    Parameters
    -------
    img : np.array
        image represented by numpy array
    np_arr: np.array
        contour with standard numpy 2d array format

    Returns
    -------
    mask_sub_img: np.array
        masked contour circumscribed sub-image

    """

    min_h, min_w = np.min(np_arr[0]), np.min(np_arr[1])
    max_h, max_w = np.max(np_arr[0]), np.max(np_arr[1])
    sub_img = img[min_h:max_h+1, min_w:max_w+1]

    shift_arr = np_arr.copy()
    shift_arr[0] -= min_h
    shift_arr[1] -= min_w
    shift_cnt = np_arr_to_cv_cnt(shift_arr)

    mask = np.zeros_like(sub_img, dtype=np.uint8)
    mask = np.ascontiguousarray(mask)
    if len(sub_img.shape) == 2:
        cv2.drawContours(mask, [shift_cnt], 0, 1, -1)
    elif len(sub_img.shape) == 3 and sub_img.shape[2] == 3:
        cv2.drawContours(mask, [shift_cnt], 0, [1, 1, 1], -1)
    else:
        raise NotImplementedError("Image shape not supported")

    mask_sub_img = sub_img * mask

    return mask_sub_img



# -*- coding: utf-8 -*-

import mahotas

from ..img import mask

__all__ = ["ZernikeMoments", ]


class ZernikeMoments:
    """ Calculate Zernike moments for contour.

    """

    def __init__(self, radius):
        """ Setting the radius

        Parameters
        -------
        radius: int
            the maximum radius for the Zernike polynomials

        """

        self.radius = radius

    def cal_fea(self, np_arr):
        """ Calculate zernike moments for contour

        Parameters
        -------
        np_arr : np.array
            contour with standard numpy 2d array format

        feas: np.array
            feature vector for image

        """

        cnt_mask = mask.build_cnt_mask(np_arr)
        feas = mahotas.features.zernike_moments(cnt_mask, self.radius)

        return feas



# -*- coding: utf-8 -*-

import numpy as np

__all__ = ["swap_wh",
           "np_arr_to_point_list",
           "point_list_to_np_arr"]

def swap_wh(np_arr):
    """ Swap row of width and row of height.

    Parameters
    -------
    np_arr : np.array
        Numpy array with height and width

    Returns
    -------
    new_arr : np.array
        Numpy array with width and height position changed

    """

    if not (len(np_arr.shape) == 2 and np_arr.shape[0] == 2):
        raise AssertionError("Wrong contour arr")

    new_arr = np.zeros_like(np_arr)
    new_arr[0] = np_arr[1]
    new_arr[1] = np_arr[0]

    return new_arr


def np_arr_to_point_list(np_arr):
    """ Convert 2d numpy array to list of points.

    Parameters
    -------
    np_arr : np.array
        Numpy array of point set

    Returns
    -------
    point_list : list
        List of points

    """

    point_list = []
    num_point = np_arr.shape[1]
    for ind in np.arange(num_point):
        point_list.append((np_arr[0][ind], np_arr[1][ind]))

    return point_list


def point_list_to_np_arr(point_list):
    """ Convert list of point coordinates to numpy 2d array.

    Parameters
    -------
    point_list : list
        list of point set ([0]-h, [1]-w)

    Returns
    -------
    np_arr : np.array
        contour with standard numpy 2d array format

    """

    num_point = len(point_list)
    np_arr = np.zeros((2, num_point))
    for ind in np.arange(num_point):
        np_arr[0][ind] = point_list[ind][0]
        np_arr[1][ind] = point_list[ind][1]

    return np_arr



# -*- coding: utf-8 -*-

from shapely.geometry import Point
from ..poly_transform import np_arr_to_poly


__all__ = ['point_in_contour', ]


def point_in_contour(np_arr, py, px):
    """ Determine point inside contour or not.

    Parameters
    -------
    np_arr : np.array
        contour with standard numpy 2d array format
    py: int
        y coordinate of point
    px: int
        x coordinate of point

    Returns
    -------
    in_flag : bool
        flag indicates point within contour or not

    """

    point = Point(px, py)
    poly = np_arr_to_poly(np_arr)

    in_flag = point.within(poly)

    return in_flag
